using cuqmbr.TravelGuide.Application.Common.Mappings;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.Addresses;

public sealed class AddressDto : IMapFrom<Address>
{
    public Guid Uuid { get; set; }

    public string Name { get; set; }

    public double Longitude { get; set; }

    public double Latitude { get; set; }

    public string VehicleType { get; set; }

    public Guid CountryUuid { get; set; }

    public string CountryName { get; set; }

    public Guid RegionUuid { get; set; }

    public string RegionName { get; set; }

    public Guid CityUuid { get; set; }

    public string CityName { get; set; }

    public void Mapping(MappingProfile profile)
    {
        profile.CreateMap<Address, AddressDto>()
            .ForMember(
                d => d.Uuid,
                opt => opt.MapFrom(s => s.Guid))
            .ForMember(
                d => d.VehicleType,
                opt => opt.MapFrom(s => s.VehicleType.Name))
            .ForMember(
                d => d.CountryUuid,
                opt => opt.MapFrom(s => s.City.Region.Country.Guid))
            .ForMember(
                d => d.CountryName,
                opt => opt.MapFrom(s => s.City.Region.Country.Name))
            .ForMember(
                d => d.RegionUuid,
                opt => opt.MapFrom(s => s.City.Region.Guid))
            .ForMember(
                d => d.RegionName,
                opt => opt.MapFrom(s => s.City.Region.Name))
            .ForMember(
                d => d.CityUuid,
                opt => opt.MapFrom(s => s.City.Guid))
            .ForMember(
                d => d.CityName,
                opt => opt.MapFrom(s => s.City.Name));
    }
}
