using cuqmbr.TravelGuide.Application.Common.Interfaces.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using FluentValidation;
using Microsoft.Extensions.Localization;

namespace cuqmbr.TravelGuide.Application.Addresses.Commands.AddAddress;

public class AddAddressCommandValidator : AbstractValidator<AddAddressCommand>
{
    public AddAddressCommandValidator(
        IStringLocalizer localizer,
        CultureService cultureService)
    {
        RuleFor(v => v.Name)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"])
            .MaximumLength(64)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.MaximumLength"],
                    64));

        RuleFor(v => v.Latitude)
            .GreaterThanOrEqualTo(-90)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.GreaterThanOrEqualTo"],
                    -90))
            .LessThanOrEqualTo(90)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.LessThanOrEqualTo"],
                    90));

        RuleFor(v => v.Longitude)
            .GreaterThanOrEqualTo(-180)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.GreaterThanOrEqualTo"],
                    -180))
            .LessThanOrEqualTo(180)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.LessThanOrEqualTo"],
                    180));

        RuleFor(v => v.VehicleType)
            .Must((v, vt) => VehicleType.Enumerations.ContainsValue(vt))
            .WithMessage(
                String.Format(
                    localizer["FluentValidation.MustBeInEnum"],
                    String.Join(
                        ", ",
                        VehicleType.Enumerations.Values.Select(e => e.Name))));

        RuleFor(v => v.CityGuid)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);
    }
}
