using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Addresses.Commands.UpdateAddress;

public class UpdateAddressCommandHandler : 
    IRequestHandler<UpdateAddressCommand, AddressDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public UpdateAddressCommandHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<AddressDto> Handle(
        UpdateAddressCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.AddressRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.City.Region.Country,
            cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        var parentEntity = await _unitOfWork.CityRepository.GetOneAsync(
            e => e.Guid == request.CityGuid, cancellationToken);

        if (parentEntity == null)
        {
            throw new NotFoundException(
                $"Parent entity with Guid: {request.CityGuid} not found.");
        }

        entity.Name = request.Name;
        entity.Longitude = request.Longitude;
        entity.Latitude = request.Latitude;
        entity.VehicleType = request.VehicleType;
        entity.CityId = parentEntity.Id;

        entity = await _unitOfWork.AddressRepository.UpdateOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();

        return _mapper.Map<AddressDto>(entity);
    }
}
