using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;

namespace cuqmbr.TravelGuide.Application.Cities.Queries.GetCitiesPage;

public class GetCitiesPageQueryHandler :
    IRequestHandler<GetCitiesPageQuery, PaginatedList<CityDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetCitiesPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<CityDto>> Handle(
        GetCitiesPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.CityRepository.GetPageAsync(
            e =>
                (e.Name.ToLower().Contains(request.Search.ToLower()) ||
                e.Region.Name.ToLower().Contains(request.Search.ToLower()) ||
                e.Region.Country.Name.ToLower().Contains(request.Search.ToLower())) &&
                (request.RegionGuid != null
                    ? e.Region.Guid == request.RegionGuid
                    : true) &&
                (request.CountryGuid != null
                    ? e.Region.Country.Guid == request.CountryGuid
                    : true),
            e => e.Region.Country,
            request.PageNumber, request.PageSize,
            cancellationToken);

        var mappedItems = _mapper
            .ProjectTo<CityDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<CityDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<CityDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
