using System.Diagnostics;
using MediatR;
using Microsoft.Extensions.Logging;

namespace cuqmbr.TravelGuide.Application.Common.Behaviours;

public class LoggingBehaviour<TRequest, TResponse> :
    IPipelineBehavior<TRequest, TResponse>
    where TRequest : notnull
{
    private readonly ILogger _logger;
    private readonly Stopwatch _stopWatch;

    public LoggingBehaviour(ILogger<TRequest> logger)
    {
        _logger = logger;
        _stopWatch = new Stopwatch();
    }

    public async Task<TResponse> Handle(
        TRequest request,
        RequestHandlerDelegate<TResponse> next,
        CancellationToken cancellationToken)
    {
        _logger.LogDebug("Started handling MediatR request.");

        _stopWatch.Start();

        var response = await next();

        _stopWatch.Stop();

        _logger.LogInformation(
            "MediatR request handled in {Duration}ms.",
            _stopWatch.ElapsedMilliseconds);

        if (_stopWatch.ElapsedMilliseconds > 500)
        {
            _logger.LogWarning(
                "MediatR request was handled slowly ({Duration}ms).",
                _stopWatch.ElapsedMilliseconds);
        }

        return response;
    }
}
