using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using PersistenceConfigurationOptions =
    cuqmbr.TravelGuide.Persistence.ConfigurationOptions;
using ApplicationConfigurationOptions =
    cuqmbr.TravelGuide.Application.ConfigurationOptions;
using IdentityConfigurationOptions =
    cuqmbr.TravelGuide.Identity.ConfigurationOptions;

namespace cuqmbr.TravelGuide.Configuration.Configuration;

public static class Configuration
{
    public static IServiceCollection ConfigureConfiguration(
        this IServiceCollection services,
        string[] args)
    {
        var environment = 
            Environment.GetEnvironmentVariable("TravelGuide_Environment");

        var configuration = new ConfigurationBuilder()
            .AddJsonFile($"./appsettings.{environment}.json", optional: true)
            .AddJsonFile($"./appsettings.json", optional: true)
            .AddEnvironmentVariables(prefix: "TravelGuide_")
            .AddCommandLine(args)
            .Build();

        services.AddOptions<PersistenceConfigurationOptions>().Bind(
            configuration.GetSection(
                PersistenceConfigurationOptions.SectionName));

        services.AddOptions<ApplicationConfigurationOptions>().Bind(
            configuration.GetSection(
                ApplicationConfigurationOptions.SectionName));

        services.AddOptions<IdentityConfigurationOptions>().Bind(
            configuration.GetSection(
                IdentityConfigurationOptions.SectionName));
        
        return services;
    }
}
