using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence.Repositories;
using cuqmbr.TravelGuide.Persistence.InMemory.Repositories;

namespace cuqmbr.TravelGuide.Persistence.InMemory;

public sealed class InMemoryUnitOfWork : UnitOfWork
{
    private readonly InMemoryDbContext _dbContext;

    public InMemoryUnitOfWork(
        InMemoryDbContext dbContext)
    {
        _dbContext = dbContext;

        CountryRepository = new InMemoryCountryRepository(_dbContext);
        RegionRepository = new InMemoryRegionRepository(_dbContext);
        CityRepository = new InMemoryCityRepository(_dbContext);
        AddressRepository = new InMemoryAddressRepository(_dbContext);
        RouteRepository = new InMemoryRouteRepository(_dbContext);
    }

    public CountryRepository CountryRepository { get; init; }

    public RegionRepository RegionRepository { get; init; }

    public CityRepository CityRepository { get; init; }

    public AddressRepository AddressRepository { get; init; }

    public RouteRepository RouteRepository { get; init; }

    public int Save()
    {
        return _dbContext.SaveChanges();
    }

    public async Task<int> SaveAsync(CancellationToken cancellationToken)
    {
        return await _dbContext.SaveChangesAsync();
    }

    public void Dispose()
    {
        Dispose(disposing: true);
        GC.SuppressFinalize(this);
    }

    public void Dispose(bool disposing)
    {
        if (disposing)
        {
            _dbContext.Dispose();
        }
    }
}
