﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

#nullable disable

namespace Persistence.PostgreSql.Migrations
{
    /// <inheritdoc />
    public partial class Add_Route_and_RouteAddresses : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.CreateSequence(
                name: "route_addresses_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "routes_id_sequence",
                schema: "application");

            migrationBuilder.CreateTable(
                name: "routes",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.routes_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    vehicle_type = table.Column<string>(type: "varchar(16)", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_routes", x => x.id);
                    table.UniqueConstraint("altk_routes_Guid", x => x.uuid);
                    table.CheckConstraint("ck_routes_vehicle_type", "vehicle_type IN ('bus', 'train', 'aircraft')");
                });

            migrationBuilder.CreateTable(
                name: "route_addresses",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.route_addresses_id_sequence')"),
                    order = table.Column<short>(type: "smallint", nullable: false),
                    address_id = table.Column<long>(type: "bigint", nullable: false),
                    route_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_route_addresses", x => x.id);
                    table.UniqueConstraint("altk_route_addresses_address_id_route_id_order", x => new { x.address_id, x.route_id, x.order });
                    table.UniqueConstraint("altk_route_addresses_Guid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_route_addresses_address_id",
                        column: x => x.address_id,
                        principalSchema: "application",
                        principalTable: "addresses",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "fk_route_addresses_route_id",
                        column: x => x.route_id,
                        principalSchema: "application",
                        principalTable: "routes",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "ix_route_addresses_address_id",
                schema: "application",
                table: "route_addresses",
                column: "address_id");

            migrationBuilder.CreateIndex(
                name: "ix_route_addresses_id",
                schema: "application",
                table: "route_addresses",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_route_addresses_route_id",
                schema: "application",
                table: "route_addresses",
                column: "route_id");

            migrationBuilder.CreateIndex(
                name: "ix_route_addresses_uuid",
                schema: "application",
                table: "route_addresses",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_routes_id",
                schema: "application",
                table: "routes",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_routes_uuid",
                schema: "application",
                table: "routes",
                column: "uuid",
                unique: true);
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "route_addresses",
                schema: "application");

            migrationBuilder.DropTable(
                name: "routes",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "route_addresses_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "routes_id_sequence",
                schema: "application");
        }
    }
}
